// src/api_bun.config.ts
var API_BUN_CONFIG = {
  LOG_LEVEL: Bun.env.LOG_LEVEL || "info",
  STORAGE: Bun.env.STORAGE || "memory",
  SECURITY: Bun.env.SECURITY || "none",
  SECRET: Bun.env.SECRET || "secret",
  API_ROOT: Bun.env.API_ROOT || "api",
  API_FORCED_TIMEOUT: Number.parseInt(Bun.env.API_FORCED_TIMEOUT || "1000")
};

// src/domain/log.service.ts
function logTrace(message, payload) {
  if (logLevel !== "verbose")
    return;
  console.log(`\uD83D\uDCD6	${message}`);
  printPayload(payload);
}
function logInfo(message, payload) {
  if (logLevel === "none")
    return;
  console.log(`\uD83D\uDCD8	${message}`);
  printPayload(payload);
}
function logWarning(message, payload) {
  if (logLevel === "none") {
    return;
  }
  console.warn(`\uD83D\uDCD2	${message}`);
  printPayload(payload);
}
function logError(message, payload) {
  if (logLevel === "none") {
    return;
  }
  console.error(`\uD83D\uDCD5	${message}`);
  printPayload(payload);
}
function logRequest(clientRequest) {
  if (logLevel !== "verbose")
    return;
  if (clientRequest.method === "OPTIONS")
    return;
  const request = getLogForRequest(clientRequest);
  console.log(`\uD83D\uDC8C	${request}`);
  printPayload(clientRequest);
}
function logResponse(clientResponse) {
  if (logLevel === "none")
    return;
  if (clientResponse.clientRequest?.method === "OPTIONS")
    return;
  const status = getLogForStatus(clientResponse);
  const request = getLogForRequest(clientResponse.clientRequest);
  console.log(`${status} ${request} `);
  printPayload(clientResponse);
}
var getLogForRequest = function(clientRequest) {
  return `${clientRequest?.method.padEnd(6, " ")} ${clientRequest?.endPoint}`;
};
var printPayload = function(payload) {
  if (!payload)
    return;
  if (logLevel !== "verbose")
    return;
  console.log(`\t${stringifyPayload(payload)}`);
  console.write("\n");
};
var stringifyPayload = function(payload) {
  if (typeof payload !== "object")
    return payload;
  try {
    return JSON.stringify(payload);
  } catch (error) {
    console.error("Error stringifying payload", error);
    return "";
  }
};
var getLogForStatus = function(clientResponse) {
  const status = clientResponse.status;
  const heart = statusColoredHearts[status] || "\uD83D\uDC94";
  const text = clientResponse.statusText || statusMessages[status] || "Unknown";
  const statusLog = `${heart}\t${status} ${text.toUpperCase().slice(0, 12).padEnd(12, " ")}`;
  return statusLog;
};
var logLevel = API_BUN_CONFIG.LOG_LEVEL;
var statusColoredHearts = {
  "200": "\uD83D\uDC9A",
  "201": "\uD83D\uDC99",
  "204": "\uD83D\uDC9C",
  "400": "\uD83D\uDC94 ",
  "401": "\uD83D\uDC98",
  "403": "\uD83D\uDC98",
  "404": "\uD83D\uDC97",
  "500": "\uD83D\uDDA4"
};
var statusMessages = {
  "200": "OK",
  "201": "Created",
  "204": "No Content",
  "400": "Bad Request",
  "401": "Unauthorized",
  "403": "Forbidden",
  "404": "Not Found",
  "500": "Server Error"
};

// src/server/client_response.class.ts
var getBodyAsString = function(body) {
  if (body === undefined) {
    return "";
  }
  if (typeof body === "object") {
    return JSON.stringify(body);
  }
  return body.toString();
};

class ClientResponse extends Response {
  bytes = 0;
  status = super.status;
  clientRequest;
  constructor(reqRes) {
    const body = reqRes.body;
    const bodyString = getBodyAsString(body);
    const options = { status: reqRes.status };
    super(bodyString, options);
    this.headers.set("Access-Control-Allow-Headers", "*");
    this.headers.set("Access-Control-Allow-Methods", "*");
    this.headers.set("Access-Control-Allow-Origin", "*");
    if (reqRes.clientRequest?.root === "api") {
      this.headers.set("Content-Type", "application/json");
    } else {
      this.headers.set("Content-Type", "text/html");
    }
    this.bytes = bodyString.length;
    this.clientRequest = reqRes.clientRequest;
    logResponse(this);
  }
}

// src/domain/crypto.service.ts
import crypto from "node:crypto";
async function hashCredentials(credentials) {
  const password = await Bun.password.hash(credentials.password);
  return { ...credentials, password };
}
async function verifyCredentials(credentials, hash) {
  return await Bun.password.verify(credentials.password, hash);
}
function generateToken(payload) {
  const sign = crypto.createHmac("sha256", secret).update(payload).digest("base64");
  return `${payload}.${sign}`;
}
function verifyToken(token) {
  const [payload, sign] = token.split(".");
  const payloadSign = crypto.createHmac("sha256", secret).update(payload).digest("base64");
  if (sign !== payloadSign) {
    return { error: "Invalid token" };
  }
  return { data: payload };
}
function generateUUID() {
  return crypto.randomUUID({});
}
var secret = API_BUN_CONFIG.SECRET;

// src/repository/file.repository.ts
async function readJson(collectionName) {
  const path = getPath(collectionName);
  try {
    const seedData = await Bun.file(path).json();
    logTrace("Seed data loaded", { path });
    return seedData;
  } catch (e) {
    logTrace("No seed data available. Starting empty collection.", { path });
    if (API_BUN_CONFIG.STORAGE === "file") {
      logInfo("Creating file storage", { path });
      writeJson(collectionName, []);
    }
    return [];
  }
}
async function writeJson(collectionName, data) {
  if (API_BUN_CONFIG.STORAGE !== "file")
    return;
  const path = getPath(collectionName);
  try {
    const content = JSON.stringify(data, null, 2);
    await Bun.write(path, content);
  } catch (e) {
    logError("Error writing to file system. Using only memory.", e);
  }
}
var getPath = function(collection) {
  return `db/${collection}.json`;
};

// src/repository/memory.repository.ts
async function selectAll(collection) {
  const result = await readCollection(collection);
  return result;
}
async function selectByKeyValue(collection, key, value) {
  const selectedData = await selectAll(collection);
  const result = selectedData.filter((item) => item[key].toString() === value);
  return result;
}
async function selectByContent(collection, content) {
  const loweredContent = content.toLowerCase();
  const selectedData = await selectAll(collection);
  const result = selectedData.filter((item) => {
    const values = Object.values(item);
    return values.some((value) => {
      if (typeof value === "string") {
        return value.toLocaleLowerCase().includes(loweredContent);
      }
      return value == content;
    });
  });
  return result;
}
async function insert(collection, item) {
  const collectionData = await readCollection(collection);
  collectionData.push(item);
  await writeCollection(collection, collectionData);
  const result = item;
  return result;
}
async function update(collection, id, item) {
  const collectionData = await readCollection(collection);
  const index = collectionData.findIndex((i) => i.id === id);
  if (index === -1) {
    return null;
  }
  collectionData[index] = item;
  await writeCollection(collection, collectionData);
  const result = collectionData[index];
  return result;
}
async function deleteById(collection, id) {
  const collectionData = await readCollection(collection);
  const index = collectionData.findIndex((i) => i.id === id);
  if (index >= 0) {
    collectionData.splice(index, 1);
    await writeCollection(collection, collectionData);
  }
}
async function readCollection(collectionName) {
  if (!db[collectionName]) {
    const fileContent = await readJson(collectionName);
    db[collectionName] = fileContent;
  }
  return [...db[collectionName]];
}
async function writeCollection(collectionName, data) {
  db[collectionName] = [...data];
  await writeJson(collectionName, data);
}
var db = {};

// src/domain/resource.service.ts
async function getAll(collection) {
  const result = await selectAll(collection);
  return result;
}
async function getById(collection, id) {
  const selectedData = await selectByKeyValue(collection, "id", id);
  const result = selectedData[0];
  const error = result ? undefined : `Item with id ${id} not found in ${collection}`;
  return { data: result, error };
}
async function getByKeyValue(collection, key, value) {
  const selectedData = await selectByKeyValue(collection, key, value);
  const result = selectedData;
  return result;
}
async function getByQueryContent(collection, queryContent) {
  const selectedData = await selectByContent(collection, queryContent);
  const result = selectedData;
  return result;
}
async function post(collection, newItem) {
  let id = newItem.id;
  if (id) {
    const existingItem = await getById(collection, id);
    if (existingItem.data) {
      return { data: undefined, error: `Item with id ${id} already exists in ${collection}` };
    }
  } else {
    id = generateUUID();
  }
  const createdAt = new Date;
  const updatedAt = null;
  const item = { ...newItem, id, createdAt, updatedAt };
  const result = await insert(collection, item);
  return { data: result };
}
async function put(collection, id, item) {
  const existingItem = await getById(collection, id);
  if (existingItem.error) {
    return { error: `Item ${id} not found on ${collection}` };
  }
  const updatedAt = new Date;
  const updatedItem = { ...existingItem.data, ...item, updatedAt };
  const updated = await update(collection, id, updatedItem);
  const result = updated ? updatedItem : undefined;
  const error = updated ? undefined : `Item ${id} not updated on ${collection}`;
  return { data: result, error };
}
async function del(collection, id) {
  await deleteById(collection, id);
}

// src/api/auth.controller.ts
async function postRegister(clientRequest) {
  const credentials = clientRequest.body;
  if (!credentials)
    return new ClientResponse({ body: "Bad request", status: 400, clientRequest });
  const email = credentials.email;
  const existingUser = await getByKeyValue("users", "email", email);
  if (existingUser.length > 0) {
    logWarning("Registering a user that already exists", { email });
    return new ClientResponse({ body: "Invalid credentials", status: 400, clientRequest });
  }
  const hashedCredentials = await hashCredentials(credentials);
  const newUser = await post("users", hashedCredentials);
  const userCredentials = castToUserCredentials(newUser.data);
  if (!userCredentials.data) {
    return new ClientResponse({ body: newUser.error, status: 500, clientRequest });
  }
  const userToken = buildUserToken(userCredentials.data);
  return new ClientResponse({ body: userToken, status: 201, clientRequest });
}
async function postLogin(clientRequest) {
  const credentials = clientRequest.body;
  const email = credentials.email;
  if (!email) {
    return new ClientResponse({ body: "Bad request", status: 400, clientRequest });
  }
  const existingUsers = await getByKeyValue("users", "email", email);
  if (!existingUsers[0]) {
    return new ClientResponse({ body: "Invalid credentials", status: 400, clientRequest });
  }
  const existingUserResult = castToUserCredentials(existingUsers[0]);
  if (!existingUserResult.data) {
    logError(existingUserResult.error || "Item malformed on db", { email });
    return new ClientResponse({ body: "Internal server error", status: 500, clientRequest });
  }
  const password = existingUserResult.data.password;
  if (!await verifyCredentials(credentials, password)) {
    return new ClientResponse({ body: "Invalid credentials", status: 404, clientRequest });
  }
  const userToken = buildUserToken(existingUserResult.data);
  return new ClientResponse({ body: userToken, status: 201, clientRequest });
}
var castToUserCredentials = function(user) {
  if (!user) {
    const error = "User not found";
    return { error };
  }
  const data = user;
  if (!data.id || !data.email || !data.password) {
    const error = "Item malformed";
    return { error };
  }
  return { data };
};
var buildUserToken = function(existingUser) {
  const id = existingUser.id;
  const email = existingUser.email;
  const user = { id, email };
  const accessToken = generateToken(id);
  const userToken = {
    user,
    accessToken
  };
  return userToken;
};

// src/api/resource.controller.ts
async function getResourceAll(resource3) {
  const selectedData = await getAll(resource3);
  const status = selectedData.length > 0 ? 200 : 200;
  const body = selectedData;
  return { status, body };
}
async function getResourceById(resource3, id) {
  const selectedData = await getById(resource3, id);
  const status = selectedData.data ? 200 : 404;
  const body = selectedData.data ? selectedData.data : selectedData.error;
  return { status, body };
}
async function getResourceByKeyValue(resource3, key, value) {
  const selectedData = await getByKeyValue(resource3, key, value);
  const status = selectedData.length > 0 ? 200 : 200;
  const body = selectedData;
  return { status, body };
}
async function getResourceByQuery(resource3, q) {
  const selectedData = await getByQueryContent(resource3, q);
  const status = selectedData.length > 0 ? 200 : 200;
  const body = selectedData;
  return { status, body };
}
async function postResource(resource3, newItem) {
  const insertedData = await post(resource3, newItem);
  const status = insertedData.data ? 201 : 409;
  const body = insertedData.data ? insertedData.data : insertedData.error;
  return { status, body };
}
async function putResource(resource3, id, item) {
  const updatedData = await put(resource3, id, item);
  const status = updatedData.data ? 200 : 404;
  const body = updatedData.data ? updatedData.data : updatedData.error;
  return { status, body };
}
async function deleteResource(resource3, id) {
  await del(resource3, id);
  const status = 204;
  const body = null;
  return { status, body };
}

// src/api/api.controller.ts
async function apiController(clientRequest) {
  if (clientRequest.resource === "") {
    return new ClientResponse({
      body: "Api root, nothing to see here. Go to /api/{resource-name} to access data.",
      status: 200,
      clientRequest
    });
  }
  switch (clientRequest.method) {
    case "GET":
      return await getController(clientRequest);
    case "POST":
      return await postController(clientRequest);
    case "PUT":
      return await putController(clientRequest);
    case "DELETE":
      return await deleteController(clientRequest);
    case "OPTIONS":
      return new ClientResponse({
        body: "",
        status: 204,
        clientRequest
      });
    default:
      return new ClientResponse({
        body: "Method not allowed",
        status: 405,
        clientRequest
      });
  }
}
async function getController(clientRequest) {
  if (clientRequest.id) {
    const myResponse = await getResourceById(clientRequest.resource, clientRequest.id);
    return new ClientResponse({ ...myResponse, clientRequest });
  }
  const params = clientRequest.params;
  if (!params) {
    const myResponse = await getResourceAll(clientRequest.resource);
    return new ClientResponse({ ...myResponse, clientRequest });
  }
  if (params.q) {
    const myResponse = await getResourceByQuery(clientRequest.resource, params.q);
    return new ClientResponse({ ...myResponse, clientRequest });
  }
  if (params.key && params.value) {
    const myResponse = await getResourceByKeyValue(clientRequest.resource, params.key, params.value);
    return new ClientResponse({ ...myResponse, clientRequest });
  }
  return new ClientResponse({ body: "Bad request", status: 400, clientRequest });
}
async function postController(clientRequest) {
  if (!clientRequest.body)
    return new ClientResponse({
      body: "Bad request. Body is required.",
      status: 400,
      clientRequest
    });
  if (clientRequest.endPoint === "/api/login") {
    return await postLogin(clientRequest);
  }
  if (clientRequest.endPoint === "/api/register") {
    return await postRegister(clientRequest);
  }
  if (clientRequest.security) {
    if (clientRequest.security.anonymous)
      return new ClientResponse({
        body: "Unauthorized. Authentication required.",
        status: 401,
        clientRequest
      });
    clientRequest.body.userId = clientRequest.security.userId;
  }
  const myResponse = await postResource(clientRequest.resource, clientRequest.body);
  return new ClientResponse({ ...myResponse, clientRequest });
}
async function putController(clientRequest) {
  if (!clientRequest.id)
    return new ClientResponse({
      body: "Bad request. Id is required to update a resource object",
      status: 400,
      clientRequest
    });
  if (clientRequest.security) {
    if (clientRequest.security.anonymous)
      return new ClientResponse({
        body: "Unauthorized, Authentication required.",
        status: 401,
        clientRequest
      });
  }
  const myResponse = await putResource(clientRequest.resource, clientRequest.id, clientRequest.body);
  return new ClientResponse({ ...myResponse, clientRequest });
}
async function deleteController(clientRequest) {
  if (!clientRequest.id)
    return new ClientResponse({
      body: "Bad request. Id is required to delete a resource object",
      status: 400,
      clientRequest
    });
  if (clientRequest.security) {
    if (clientRequest.security.anonymous)
      return new ClientResponse({
        body: "Unauthorized. Authentication required.",
        status: 401,
        clientRequest
      });
  }
  const myResponse = await deleteResource(clientRequest.resource, clientRequest.id);
  return new ClientResponse({ ...myResponse, clientRequest });
}

// src/api/forced.controller.ts
async function forcedController(clientRequest) {
  let status = clientRequest.force?.status || 500;
  if (status > 599 || status < 100) {
    status = 500;
  }
  const body = getBody(clientRequest);
  return new ClientResponse({ body, status, clientRequest });
}
var getBody = function(clientRequest) {
  if (clientRequest.body)
    return clientRequest.body;
  const forcedItem = {
    id: clientRequest.id || 0,
    message: "Forced response",
    timestamp: new Date().toISOString(),
    force: clientRequest.force
  };
  if (clientRequest.id)
    return forcedItem;
  const forcedArray = [forcedItem];
  return forcedArray;
};

// src/api/web.controller.ts
async function getWebController(clientRequest) {
  let myResponse = getNotFoundWebResponse(clientRequest);
  if (clientRequest.endPoint === "/") {
    myResponse = getHomePageWebResponse();
  }
  return new ClientResponse({ ...myResponse, clientRequest });
}
var getHomePageWebResponse = function() {
  const body = `
    <!DOCTYPE html>
      <html lang="en" >
        <head>
          <meta charset="UTF-8">
          <meta name="viewport" content="width=device-width, initial-scale=1">
          <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/@picocss/pico@2.0.6/css/pico.classless.min.css"/>
          <title>\uD83E\uDD56 API Bun</title>
        </head>
        <body>
          <header>  
            <h1>\uD83E\uDD56<a href="https://github.com/AlbertoBasalo/api_bun" target="_blank" >API Bun</a></h1>
            <h3>\u2615 Your delicious and fast Rest API ready to consume.</h3>
          </header>
          <main >
            <ul>
              <li>For rapid prototypes.</li>
              <li>For small pet projects.</li>
              <li>For educational purposes.</li>
              <li>Read the <a href="https://github.com/AlbertoBasalo/api_bun/blob/main/README.md" target="_blank" >docs</a> for more information.</li>
              <li>Check the <a href="./api/activities">activities</a> sample endpoint.</li>
            </ul>
          </main>
          <footer>
            <blockquote>
              \uD83E\uDDD1\u200D\uD83C\uDF73 Cooked with <a href="https://bun.sh" target="_blank">bun</a> and love
              <footer><cite>by <a href="https://albertobasalo.dev" target="_blank" >Alberto Basalo</a></cite></footer>
            </blockquote>
          </footer>
        </body>
      </html>
    `;
  return {
    body,
    status: 200
  };
};
var getNotFoundWebResponse = function(clientRequest) {
  const body = `Not found: ${clientRequest.endPoint}`;
  const status = 404;
  return { body, status };
};

// src/server/client_request.service.ts
async function buildClientRequest(request) {
  const method = request.method;
  const url = new URL(request.url);
  const endPoint = url.pathname;
  const query = url.searchParams;
  const { root, resource: resource4, id } = extractEndPoint(endPoint);
  const body = await extractBody(request);
  const params = extractParams(query);
  const force = await extractForce(query);
  const security = extractSecurity(request);
  const clientRequest = {
    method,
    endPoint,
    root,
    resource: resource4,
    id,
    params,
    body,
    force,
    security
  };
  logRequest(clientRequest);
  return clientRequest;
}
var extractEndPoint = function(endPoint) {
  const splits = endPoint.split("/");
  const root = splits[1] || "";
  const resource4 = splits[2] || "";
  const id = splits[3];
  return { root, resource: resource4, id };
};
var extractParams = function(query) {
  const q = query.get("q") || undefined;
  const key = query.get("key") || undefined;
  const value = query.get("value") || undefined;
  const sort = query.get("sort") || undefined;
  const order = query.get("order") || undefined;
  if (!q && !key && !value && !sort && !order)
    return;
  return { q, key, value, sort, order };
};
async function extractBody(request) {
  if (request.method === "POST" || request.method === "PUT") {
    const body = await request.json();
    const props = Object.keys(body).length;
    if (props === 0)
      return;
    return body;
  }
  return;
}
async function extractForce(query) {
  const qDelay = query.get("delay");
  let delay;
  if (qDelay)
    delay = Number.parseInt(qDelay);
  const qStatus = query.get("status");
  let status;
  if (qStatus)
    status = Number.parseInt(qStatus);
  if (delay === undefined && status === undefined) {
    return;
  }
  return { delay, status };
}
var extractSecurity = function(request) {
  if (API_BUN_CONFIG.SECURITY === "none")
    return;
  const security = { anonymous: true, userId: undefined };
  const auth2 = request.headers.get("Authorization");
  if (!auth2)
    return security;
  const accessToken = auth2.split(" ")[1];
  if (!accessToken)
    return security;
  const user = verifyToken(accessToken);
  if (user.error) {
    logTrace("Extracting UserId", user.error);
    return security;
  }
  security.userId = user.data;
  security.anonymous = false;
  return security;
};

// src/server/request.handler.ts
async function handleRequest(request) {
  const clientRequest = await buildClientRequest(request);
  await handleDelayedResponse(clientRequest.force?.delay);
  if (clientRequest.force?.status) {
    return await forcedController(clientRequest);
  }
  if (clientRequest.root === API_BUN_CONFIG.API_ROOT) {
    return await apiController(clientRequest);
  }
  return await getWebController(clientRequest);
}
async function handleDelayedResponse(delayMs) {
  if (!delayMs)
    return;
  logTrace(`Forced delaying response by ${delayMs} ms`);
  await new Promise((resolve) => setTimeout(resolve, delayMs));
}

// src/index.ts
var getErrorMessage = function(error) {
  return error instanceof Error ? error.message : "Internal server error";
};
var isFavIcon = function(req) {
  return new URL(req.url).pathname === "/favicon.ico";
};
var main = function() {
  console.clear();
  logInfo(`API-bun is listening on ${server.url} \uD83E\uDD56`, API_BUN_CONFIG);
  const recipe = {
    bun: "https://bun.sh",
    baker: "https://twitter.com/AlbertoBasalo"
  };
  logInfo("Delicious JSON fast food ready to consume. \u2615");
  logInfo(`Cooked with ${recipe.bun} \uD83E\uDEF6; by ${recipe.baker} \uD83E\uDDD1\u200D\uD83C\uDF73`, recipe);
  const sampleEndPoint = `${server.url}api/activities`;
  logInfo(`Try the sample ${sampleEndPoint} end point \uD83C\uDF73`);
};
var server = Bun.serve({
  port: 3000,
  fetch(req) {
    try {
      if (isFavIcon(req))
        return new Response(null, { status: 204 });
      return handleRequest(req);
    } catch (error) {
      logError("Fatal Error processing request", { error, req });
      const message = getErrorMessage(error);
      return new Response(message, { status: 500 });
    }
  }
});
main();
